/*
 * This file is a part of VyQChat.
 *
 * Copyright (C) 2002-2003 Pawel Stolowski <yogin@linux.bydg.org>
 *
 * VyQChat is free software; you can redestribute it and/or modify it
 * under terms of GNU General Public License by Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY. See GPL for more details.
 */

#ifndef __SETTINGS_H
#define __SETTINGS_H

#include "sound.h"
#include <qtabdialog.h>

class QListBox;
class QLineEdit;
class QCheckBox;
class QComboBox;
class QSpinBox;
class QVButtonGroup;

class Settings
{
	private:
		QString nick, def_nick;
		QString hostname, def_hostname;
		QString ip, def_ip;
		QString broadcast, def_broadcast;
		QString topic, def_topic;
		Sound::SoundType def_sound;
		int refresh;
		QColor special_color;
		QColor normal_color;
		QColor own_color;
		int x, y, width, height, listwidth;
		Sound::SoundType sound;
		QString playcmd;
		QString snd[Sound::Dummy];
		bool sndp[Sound::Dummy]; //play-dont play flags
		QString def_snd[Sound::Dummy];
		QString def_playcmd;
		bool hide_channels;
		bool regexp_on;
		bool toolbar_on;
		bool userslist_on;
		bool popup;
		bool nopopup_dnd;
		//int encoding;
		QString encoding;

	public:
		Settings();
		~Settings();
		bool getDefaults();
		void load();
		void save();
	
		friend class MainWindow;
		friend class SettingsDialog;
		friend class Sound;
		friend class SoundSystem;
		friend class SoundCommand;
};

class SettingsDialog: public QTabDialog
{
	Q_OBJECT
	private slots:
		void slotApply();
		void slotDefaults();
		void slotPopupCheckBoxChanged(bool state);
		void slotSoundEventSelected(int index);
		void slotBrowseSoundClicked();
		void slotSoundSystemClicked(int index);
		void slotPlayFlagClicked(bool state);
	private:
		QLineEdit *le_nick, *le_hostname, *le_ip, *le_broadcast, *le_sndname,
			*le_playcmd;
		QSpinBox *sp_refresh;
		QVButtonGroup *opt_snddev;
		QCheckBox *b_hide, *b_popup, *b_popupdnd, *b_play;
		QListBox *snd_list;
		QComboBox *l_enc;
		Settings settings;  //copy of settings for manipulation
		Settings *currsettings; //current application settings
		
		void setupNetworkTab(QWidget *p);
		void setupSoundDeviceTab(QWidget *p);
		void setupSoundsTab(QWidget *p);
		void setupLookTab(QWidget *p);
		void setupMiscTab(QWidget *p);
	public:
		SettingsDialog(QWidget *parent, Settings *s);
		~SettingsDialog();
};

#endif

