/*
 * This file is a part of VyQChat.
 *
 * Copyright (C) 2002-2003 Pawel Stolowski <yogin@linux.bydg.org>
 *
 * VyQChat is free software; you can redestribute it and/or modify it
 * under terms of GNU General Public License by Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY. See GPL for more details.
 */

#include "vysendsocket.h"
#include "vypacket.h"
#include <stdlib.h>
#include <stdarg.h>
#include <stdio.h>
#include <string.h>
#include <time.h>
#include <sys/socket.h>

VySendSocket::VySendSocket(const QString &addr, QTextCodec *c): QSocketDevice(QSocketDevice::Datagram), tc(c)/*{{{*/
{
	if (broadcast.setAddress(addr))
	{	
		int opt = 1;
		setBlocking(FALSE);
		setsockopt(socket(), SOL_SOCKET, SO_BROADCAST, &opt, sizeof(int)); 
	}
	else
	{
		qWarning("error setting IP address");
	}
	pdata = new char [VY_MAXPACKETSIZE+1];
	*pdata = 'X';
	plen = 0;
	srandom(time(NULL));
}/*}}}*/

VySendSocket::~VySendSocket()/*{{{*/
{
	delete pdata;
}/*}}}*/

void VySendSocket::setTextCodec(QTextCodec *c)/*{{{*/
{
	tc = c;
}/*}}}*/

void VySendSocket::send(const QHostAddress &addr, const char *format, ...)/*{{{*/
{
	va_list arg;
	va_start(arg, format);
	plen = 0;
	//
	// packet id
	for (int i=1; i<11; i++)
		pdata[i] = 0x20 + random()%95;
	//
	// the rest of the packet
	plen = vsprintf(pdata+10, format, arg);
	va_end(arg);
	plen += 11;
	//
	// send it to the network
	int went = writeBlock(pdata, plen, addr, VY_PORT);
	if (went != plen)
	{
		qWarning("socket overfull... %d", (int)error());
	}
}/*}}}*/

void VySendSocket::vy_say(const QString &channel, const QString &nick, const QString &text)/*{{{*/
{
	send(broadcast, "%c%s%c%s%c%s%c", VY_SAY, 
			static_cast<const char*>(tc->fromUnicode(channel)), 0,
			static_cast<const char*>(tc->fromUnicode(nick)), 0,
			static_cast<const char*>(tc->fromUnicode(text)), 0);
}/*}}}*/

void VySendSocket::vy_me(const QString &channel, const QString &nick, const QString &text)/*{{{*/
{
	send(broadcast, "%c%s%c%s%c%s%c", VY_ME, 
			static_cast<const char*>(tc->fromUnicode(channel)), 0,
			static_cast<const char*>(tc->fromUnicode(nick)), 0,
			static_cast<const char*>(tc->fromUnicode(text)), 0);
}/*}}}*/

void VySendSocket::vy_whohere(const QString &updater)/*{{{*/
{
	send(broadcast, "%c%s%c", VY_WHOHERE, 
			static_cast<const char*>(tc->fromUnicode(updater)), 0);
}/*}}}*/

void VySendSocket::vy_imhere(const QString &updater, const QString &responder, char status, char winstate)/*{{{*/
{
	send(broadcast, "%c%s%c%s%c%c%c", VY_IMHERE, 
			static_cast<const char*>(tc->fromUnicode(updater)), 0,
			static_cast<const char*>(tc->fromUnicode(responder)), 0,
			status, winstate);
}/*}}}*/

void VySendSocket::vy_beep(const QHostAddress &addr, const QString& to, const QString& from)/*{{{*/
{
	send(addr, "%c%c%s%c%s%c", VY_BEEP, VY_BEEP_SIGNAL,
			static_cast<const char*>(tc->fromUnicode(to)), 0,
			static_cast<const char*>(tc->fromUnicode(from)), 0);
}/*}}}*/

void VySendSocket::vy_beepack(const QHostAddress &addr, const QString& to, const QString& from, char gender)/*{{{*/
{
	send(addr, "%c%c%s%c%s%c%c", VY_BEEP, VY_BEEP_ACK, 
			static_cast<const char*>(tc->fromUnicode(from)), 0,
			static_cast<const char*>(tc->fromUnicode(to)), 0, gender);
}/*}}}*/

void VySendSocket::vy_join(const QString &who, const QString &channel, char status, char gender)/*{{{*/
{
	send(broadcast, "%c%s%c%s%c%c%c", VY_JOIN, 
			static_cast<const char*>(tc->fromUnicode(who)), 0,
			static_cast<const char*>(tc->fromUnicode(channel)), 0, status, gender);
}/*}}}*/

void VySendSocket::vy_leave(const QString &who, const QString &channel, char gender)/*{{{*/
{
	send(broadcast, "%c%s%c%s%c%c", VY_LEAVE, static_cast<const char*>(tc->fromUnicode(who)), 0,
			static_cast<const char*>(tc->fromUnicode(channel)), 0,
			gender);
}/*}}}*/

void VySendSocket::vy_private_join(const QHostAddress &addr, const QString &from, const QString &to, char gender)/*{{{*/
{
	send(addr, "%c%c%s%c%s%c%c", VY_PRIVATE, VY_PRIVATE_JOIN, static_cast<const char*>(tc->fromUnicode(from)), 0, static_cast<const char*>(tc->fromUnicode(to)), 0, gender);
}/*}}}*/

void VySendSocket::vy_private_leave(const QHostAddress &addr, const QString &from, const QString &to, char gender)/*{{{*/
{
	send(addr, "%c%c%s%c%s%c%c", VY_PRIVATE, VY_PRIVATE_LEAVE, static_cast<const char*>(tc->fromUnicode(from)), 0, static_cast<const char*>(tc->fromUnicode(to)), 0, gender);
}/*}}}*/

void VySendSocket::vy_private_say(const QHostAddress &addr, const QString &from, const QString &to, const QString &text)/*{{{*/
{
	send(addr, "%c%c%s%c%s%c%s%c", VY_PRIVATE, VY_PRIVATE_SAY, static_cast<const char*>(tc->fromUnicode(from)), 0, static_cast<const char*>(tc->fromUnicode(to)), 0, static_cast<const char*>(tc->fromUnicode(text)), 0);
}/*}}}*/

void VySendSocket::vy_private_me(const QHostAddress &addr, const QString &from, const QString &to, const QString &text)/*{{{*/
{
	send(addr, "%c%c%s%c%s%c%s%c", VY_PRIVATE, VY_PRIVATE_ME, static_cast<const char*>(tc->fromUnicode(from)), 0, static_cast<const char*>(tc->fromUnicode(to)), 0, static_cast<const char*>(tc->fromUnicode(text)), 0);
}/*}}}*/

void VySendSocket::vy_message(const QHostAddress &addr, const QString &from, const QString &to, const QString &text)/*{{{*/
{
	send(addr, "%c%s%c%s%c%s%c", VY_MESSAGE, static_cast<const char*>(tc->fromUnicode(from)), 0, static_cast<const char*>(tc->fromUnicode(to)), 0, static_cast<const char*>(tc->fromUnicode(text)), 0);
}/*}}}*/

void VySendSocket::vy_massmessage(const QHostAddress &addr, const QString &from, const QString &to, const QString &text)/*{{{*/
{
	send(addr, "%c%s%c%s%c%s%c", VY_MASSMESSAGE, static_cast<const char*>(tc->fromUnicode(from)), 0, static_cast<const char*>(tc->fromUnicode(to)), 0, static_cast<const char*>(tc->fromUnicode(text)), 0);
}/*}}}*/

void VySendSocket::vy_messageack(const QHostAddress &addr, char status, const QString &from, const QString &to, char gender, const QString &answer)/*{{{*/
{
	send(addr, "%c%c%s%c%s%c%c%s%c", VY_MESSAGEACK, status, static_cast<const char*>(tc->fromUnicode(from)), 0, static_cast<const char*>(tc->fromUnicode(to)), 0, gender, static_cast<const char*>(tc->fromUnicode(answer)), 0);
}/*}}}*/

void VySendSocket::vy_herereq(const QString &from, const QString &channel)/*{{{*/
{
	send(broadcast, "%c%s%c%s%c", VY_HEREREQ, static_cast<const char*>(tc->fromUnicode(from)), 0, static_cast<const char*>(tc->fromUnicode(channel)), 0);
}/*}}}*/

void VySendSocket::vy_hereack(const QHostAddress &addr, const QString &to, const QString &channel, const QString &from, char ra)/*{{{*/
{
	send(addr, "%c%s%c%s%c%s%c", VY_HEREACK, static_cast<const char*>(tc->fromUnicode(to)), 0, static_cast<const char*>(tc->fromUnicode(channel)), 0, static_cast<const char*>(tc->fromUnicode(from)), 0, ra);
}/*}}}*/

void VySendSocket::vy_inforeq(const QHostAddress &addr, const QString &to, const QString &from)/*{{{*/
{
	send(addr, "%c%s%c%s%c", VY_INFOREQ, static_cast<const char*>(tc->fromUnicode(to)), 0, static_cast<const char*>(tc->fromUnicode(from)), 0);
}/*}}}*/

void VySendSocket::vy_inforeqack(const QHostAddress &addr, const QString &to, const QString &from, const QString &comp,	const QString &name, const QString &ip, const QString &channels, const QString &answer)/*{{{*/
{
	//send(addr, "%c%s%c%s%c%s%c%s%c%s%c%s#%c%s%c", VY_INFOREQACK, static_cast<const char*>(tc->fromUnicode(to)), 0,
	send(addr, "%c%s%c%s%c%s%c%s%c%s%c%s%c%s%c", VY_INFOREQACK, static_cast<const char*>(tc->fromUnicode(to)), 0,
			static_cast<const char*>(tc->fromUnicode(from)), 0, static_cast<const char*>(tc->fromUnicode(comp)), 0,
			static_cast<const char*>(tc->fromUnicode(name)), 0, static_cast<const char*>(tc->fromUnicode(ip)), 0,
			static_cast<const char*>(tc->fromUnicode(channels)), 0, static_cast<const char*>(tc->fromUnicode(answer)), 0);
}/*}}}*/

void VySendSocket::vy_nickchange(const QString &from, const QString &to, char gender)/*{{{*/
{
	send(broadcast, "%c%s%c%s%c%c", VY_NICKCHANGE, 
			static_cast<const char*>(tc->fromUnicode(from)), 0,
			static_cast<const char*>(tc->fromUnicode(to)), 0, gender);
}/*}}}*/

void VySendSocket::vy_changestatus(const QString &who, char status, char gender, const QString &answer)/*{{{*/
{
	send(broadcast, "%c%s%c%c%c%s%c", VY_CHANGESTAT, 
			static_cast<const char*>(tc->fromUnicode(who)), 0,
			status, gender,
			static_cast<const char*>(tc->fromUnicode(answer)), 0);
}/*}}}*/

void VySendSocket::vy_newtopic(const QString &channel, const QString &topic, const QString &who)/*{{{*/
{
	send(broadcast, "%c%s%c%s (%s)%c", VY_NEWTOPIC, static_cast<const char*>(tc->fromUnicode(channel)), 0, static_cast<const char*>(tc->fromUnicode(topic)), static_cast<const char*>(tc->fromUnicode(who)), 0);
}/*}}}*/

void VySendSocket::vy_currenttopic(const QHostAddress &addr, const QString &to, const QString &channel, const QString &topic)/*{{{*/
{
	send(addr, "%c%s%c%s%c%s%c", VY_CURRTOPIC, static_cast<const char*>(tc->fromUnicode(to)), 0, static_cast<const char*>(tc->fromUnicode(channel)), 0, static_cast<const char*>(tc->fromUnicode(topic)), 0);
}/*}}}*/

